#!/usr/bin/ruby
# encoding: utf-8

require 'iconv'
require 'gtk2'
require 'vte'
require "rexml/document"
require 'optparse'
require 'sqlite3'

require "MfsDiskDrive.rb"
require "MfsSinglePartition.rb"
require "AntibotDriveScreen.rb"
require "AntibotOptionScreen.rb"
require "AntibotScanFrame.rb"
require "AntibotPrereqScreen.rb"
require "AntibotFinishFrame.rb"
require "AntibotAviraScanner.rb" 
require "AntibotResultFrame.rb"
require "AntibotInfectedFile.rb"
require "AntibotMisc.rb"

# Create a new assistant widget with no pages. */
assistant = Gtk::Assistant.new
assistant.set_size_request(600, 350)   # 450, 300
assistant.signal_connect('destroy') { Gtk.main_quit }

# Append the page for checking prerequisites
prescr = AntibotPrereqScreen.new(assistant)
assistant.append_page(prescr.wdgt)
assistant.set_page_title(prescr.wdgt, "Voraussetzungen für Virenscan")
assistant.set_page_type(prescr.wdgt, Gtk::Assistant::PAGE_INTRO)
assistant.set_page_complete(prescr.wdgt, prescr.complete)

# Append the page for drive selection
dscrn = AntibotDriveScreen.new(assistant)
assistant.append_page(dscrn.wdgt) 
assistant.set_page_title(dscrn.wdgt, "Laufwerke auswählen")
assistant.set_page_complete(dscrn.wdgt, true)

# Append the page for scan options
oscrn = AntibotOptionScreen.new
# Append the page for scan start
scnfr = AntibotScanFrame.new(assistant, dscrn, oscrn, prescr, Time.now)
assistant.append_page(scnfr.wdgt)
assistant.set_page_title(scnfr.wdgt, "Virenscan starten")
assistant.set_page_type(scnfr.wdgt, Gtk::Assistant::PAGE_PROGRESS)
assistant.set_page_complete(scnfr.wdgt, false)

# Append the results page
resfr = AntibotResultFrame.new(assistant)
assistant.append_page(resfr.wdgt)
assistant.set_page_title(resfr.wdgt, "Auswertung")
assistant.set_page_type(resfr.wdgt, Gtk::Assistant::PAGE_SUMMARY)
assistant.set_page_complete(resfr.wdgt, false)

# Append the finish page - shown after desinfection
finfr = AntibotFinishFrame.new(assistant)

# Current page for re-reading drivelist

assistant.signal_connect("prepare") { |s, p|
	resfr.again_button.hide_all
	if p == dscrn.wdgt
		puts "Moving to drivelist"
		dscrn.reread_drivelist
	elsif p == scnfr.wdgt
		puts "Moving to scan"
	elsif p == resfr.wdgt
		puts "Jumping to results page"
		resfr.show_results(dscrn.int_drives, dscrn.ext_drives, scnfr.scanner)
	end
	dscrn.dump_marked
}

assistant.signal_connect("close") {
	puts "Closing. Goodbye."
	( dscrn.ext_drives + dscrn.int_drives ).each { |d| d.umount } 
	system("touch /var/run/lesslinux/reboot_requested")
	AntibotMisc.cancel_dialog(assistant)
}

assistant.signal_connect("cancel") {
	puts "Cancelled! Goodby."
	( dscrn.ext_drives + dscrn.int_drives ).each { |d| d.umount } 
	system("touch /var/run/lesslinux/restart_requested")
	AntibotMisc.cancel_dialog(assistant)
}

assistant.deletable = false
assistant.decorated = false
assistant.window_position = Gtk::Window::POS_CENTER_ALWAYS
assistant.show_all
Gtk.main
